﻿using System;
using System.ComponentModel.Composition;
using ParkingLot_MEF.Interfaces;

namespace ParkingLot_MEF.Parkoviste
{

    [Export(typeof(IPrijezdyOdjezdy))]
    public class Parkoviste : IPrijezdyOdjezdy
    {
        private const int MistaAuto = 1;
        private const int MistaBus = 2;
        private const int MistaTir = 4;
        private int _pocetAuta;
        private int _pocetBus;
        private int _pocetTir;
        private int _volnaMista;
        
        [Import]
        IKonfigurace Konfigurace { get; set; }

       [ImportingConstructor]
        public Parkoviste(IKonfigurace konfigurace)
        {
            _volnaMista = konfigurace.GetKapacita();
            _pocetAuta = 0;
            _pocetBus = 0;
            _pocetTir = 0;
        }

        public void OdjelBus()
        {
            if (((_volnaMista + MistaBus) <= Konfigurace.GetKapacita()) && _pocetBus > 0)
            {
                _volnaMista += MistaBus;

                _pocetBus--;
                PosliZpravuMista();
            }
            else
            {
                throw new Exception("Neni bus k odjezdu!");
            }
        }

        public void OdjelTir()
        {
            if (((_volnaMista + MistaTir) <= Konfigurace.GetKapacita()) && _pocetTir > 0)
            {
                _volnaMista += MistaTir;

                _pocetTir--;
                PosliZpravuMista();
            }
            else
            {
                throw new Exception("Neni tir k odjezdu!");
            }
        }

        public void OdjeloAuto()
        {
            if (((_volnaMista + MistaAuto) <= Konfigurace.GetKapacita()) && _pocetAuta > 0)
            {
                _volnaMista += MistaAuto;

                _pocetAuta--;
                PosliZpravuMista();
            }
            else
            {
                throw new Exception("Neni auto k odjezdu!");
            }
        }

        public void PrijelBus()
        {
            if ((_volnaMista - MistaBus) >= 0)
            {
                _volnaMista -= MistaBus;

                _pocetBus++;
                PosliZpravuMista();
            }
            else
            {
                if (_volnaMista == 0)
                {
                    PosliZpravuMista();
                }

                throw new Exception("Nedostatek mist pro bus!");
            }
        }

        public void PrijelTir()
        {
            if ((_volnaMista - MistaTir) >= 0)
            {
                _volnaMista -= MistaTir;

                _pocetTir++;
                PosliZpravuMista();
            }
            else
            {
                if (_volnaMista == 0)
                {
                    PosliZpravuMista();
                }

                throw new Exception("Nedostatek mist pro tir!");
            }
        }

        public void PrijeloAuto()
        {
            if ((_volnaMista - MistaAuto) >= 0)
            {
                _volnaMista -= MistaAuto;

                _pocetAuta++;
                PosliZpravuMista();
            }
            else
            {
                PosliZpravuMista();
                throw new Exception("Nedostatek mist pro auto!");
            }
        }

        private void PosliZpravuMista()
        {
            ParkovisteZbyvaEvent.Instance.PublishZbyvaEvent(this, _volnaMista);
            if (_volnaMista == 0)
            {
                PosliZpravuPlno();
            }
        }

        private void PosliZpravuPlno()
        {
            ParkovistePlneEvent.Instance.PublishPlnoEvent(this);
        }
    }
}