﻿using System;
using System.ComponentModel.Composition;
using System.Threading;
using ParkingLot_MEF.Interfaces;
using ParkingLot_MEF.Parkoviste;

namespace ParkingLot_MEF.Pult
{
    internal class Pult
    {
        [Import]
        public IZapVyp Tabule { get; set; }

        [Import("Brana")]
        public IZapVyp Brana { get; set; }

        [Import("Brana")]
        public IPrijeloOdjelo Statistika { get; set; }

        public int PracovniDoba;

        public Pult()
        {
            PracovniDoba = Properties.Settings.Default.PracovniDoba;
            Subscribe();
        }

        public void Subscribe()
        {
            ParkovisteZbyvaEvent.Instance.Event += ParkovisteZbyvaEventHandling;
            ParkovistePlneEvent.Instance.Event += ParkovistePlneEventHandling;
        }

        private static void ParkovisteZbyvaEventHandling(Object source, int volnaMista)
        {
            Console.Out.WriteLine("(Pult) Volna mista: " + volnaMista);
        }

        private static void ParkovistePlneEventHandling(Object source)
        {
            Console.Out.WriteLine("(Pult) Parkoviste je plne!");
        }

        public void Zacatek()
        {
            Console.Out.WriteLine("Pracovni doba zacala, spoustim tabuli a branu...");
            Tabule.Zap();
            Brana.Zap();
        }

        public void Konec()
        {
            Console.Out.WriteLine("Pracovni doba konci, vypinam tabuli a branu...");
            Tabule.Vyp();
            Brana.Vyp();
            Console.Out.WriteLine("Na parkovisti zustalo: " + (Statistika.GetPrijelo() - Statistika.GetOdjelo()) +
                                  " vozidel...");
        }

        public static void OpakujPracovniDobu(Pult pult)
        {
            while (true)
            {
                pult.Zacatek();
                try
                {
                    Thread.Sleep(pult.PracovniDoba);
                }
                catch (Exception e)
                {
                    Console.Out.WriteLine(e);
                }
                pult.Konec();
            }
        }
    }
}